@@ -2,9 +2,10 @@
 
 package XML::Atom::Content;
 use strict;
-
-use XML::Atom;
 use base qw( XML::Atom::ErrorHandler );
+
+use Encode;
+use XML::Atom;
 use XML::Atom::Util qw( set_ns remove_default_ns hack_unicode_entity );
 use MIME::Base64 qw( encode_base64 decode_base64 );
 
@@ -69,10 +70,11 @@ sub body {
             $elem->removeChild($_) for $elem->getChildNodes;
         }
         if (!_is_printable($data)) {
+            my $raw = Encode::encode("utf-8", $data);
             if (LIBXML) {
-               $elem->appendChild(XML::LibXML::Text->new(encode_base64($data, '')));
+               $elem->appendChild(XML::LibXML::Text->new(encode_base64($raw, '')));
             } else {
-               $elem->appendChild(XML::XPath::Node::Text->new(encode_base64($data, '')));
+               $elem->appendChild(XML::XPath::Node::Text->new(encode_base64($raw, '')));
             }
             $elem->setAttribute('mode', 'base64');
         } else {
@@ -123,16 +125,17 @@ sub body {
                 } else {
                     $content->{__body} = LIBXML ? $elem->textContent : $elem->string_value;
                 }
+                if ($] >= 5.008) {
+                    $content->{__body} = hack_unicode_entity($content->{__body});
+                }
             } elsif ($mode eq 'base64') {
-                $content->{__body} = decode_base64(LIBXML ? $elem->textContent : $elem->string_value);
+                my $raw = decode_base64(LIBXML ? $elem->textContent : $elem->string_value);
+                $content->{__body} = Encode::decode("utf-8", $raw);
             } elsif ($mode eq 'escaped') {
                 $content->{__body} = LIBXML ? $elem->textContent : $elem->string_value;
             } else {
                 $content->{__body} = undef;
             }
-            if ($] >= 5.008) {
-                $content->{__body} = hack_unicode_entity($content->{__body});
-            }
         }
     }
     $content->{__body};
@@ -141,8 +144,13 @@ sub body {
 sub _is_printable {
     my $data = shift;
 
-    # printable ASCII or UTF-8 bytes
-    $data =~ /^(?:[\x09\x0a\x0d\x20-\x7f]|[\xc0-\xdf][\x80-\xbf]|[\xe0-\xef][\x80-\xbf][\x80-\xbf])*$/;
+    # try decoding this $data with UTF-8
+    my $decoded =
+        ( Encode::is_utf8($data)
+          ? $data
+          : eval { Encode::decode("utf-8", $data, Encode::FB_CROAK) } );
+
+    return ! $@ && $decoded =~ /^\p{IsPrint}*$/;
 }
 
 sub as_xml {
