@@ -11,22 +11,32 @@ use Exporter;
 @ISA = qw( Exporter );
 
 our %NS_MAP = (
-    0.3 => 'http://purl.org/atom/ns#',
-    1.0 => 'http://www.w3.org/2005/Atom',
+    '0.3' => 'http://purl.org/atom/ns#',
+    '1.0' => 'http://www.w3.org/2005/Atom',
 );
 
+our %NS_VERSION = reverse %NS_MAP;
+
 sub set_ns {
     my $thing = shift;
     my($param) = @_;
-    if ($param->{Namespace}) {
-        $thing->{ns} = $param->{Namespace};
+    if (my $ns = delete $param->{Namespace}) {
+        $thing->{ns}      = $ns;
+        $thing->{version} = $NS_VERSION{$ns};
     } else  {
-        my $version = $param->{Version} || '0.3';
+        my $version = delete $param->{Version} || '0.3';
+        $version    = '1.0' if $version == 1;
         my $ns = $NS_MAP{$version} or $thing->error("Unknown version: $version");
         $thing->{ns} = $ns;
+        $thing->{version} = $version;
     }
 }
 
+sub ns_to_version {
+    my $ns = shift;
+    $NS_VERSION{$ns};
+}
+
 sub hack_unicode_entity {
     my $data = shift;
     Encode::_utf8_on($data);
